/* UnQuill: Disassemble games written with the "Quill" adventure game system
    Copyright (C) 1996,1999  John Elliott

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "unquill.h"

void zcode_binary(void)
{
	FILE *fp = fopen("iqrun.z5", "rb");
	char achad[5];
	ushrt n;
	int i;

#ifdef LIBDIR
	char fname[PATH_MAX];

	if (!fp)
	{
		strcpy(fname, LIBDIR);
		strcat(fname, "iqrun.z5");
		fp = fopen(fname, "rb");
	}
	if (!fp)
	{
		fprintf(stderr,"Cannot open iqrun.z5 or %siqrun.z5\n",
			LIBDIR);
		exit(1);

	}

#else
	if (!fp) 
	{
		fprintf(stderr,"Cannot open iqrun.z5\n");
		exit(1);	
	}
#endif
	/* Firstly: Copy bytes from Z-code file to outfile, until we see
         * the magic word ACHAD - the spell in Heavy On The Magick which
         * will revive a long-dead being. As Apex puts it so well:
         *
         * FOR AI IS DEAD 
         * TAKE ARM LEG HEAD
         * IN POT DISPLAY
         * AND ONE WORD SAY
         *
         */
	for (n = 0; n < 5; n++) achad[n] = 0;

	while (!feof(fp))
	{
		for (n = 0; n < 4; n++) achad[n] = achad[n+1];
		achad[4] = fgetc(fp);
		fputc(achad[4], outfile);
		if (memcmp(achad, "Achad", 5)) continue;

	/* We've got it. That's dealt with the head, so let's do the arm.
         * 
         */
		for (n = 0; n < 0xA400; n++)
		{
			fgetc(fp); /* Skip the corresponding z-code byte */
			fputc(zmem(n + 0x5C00), outfile);
		}
	/* Now the options */
		fgetc(fp); fputc(1,     outfile);	/* Valid game */
		fgetc(fp); fputc(islql, outfile); /* Late or early? */
		fgetc(fp); fputc(mopt,  outfile); /* Mono? */
		fgetc(fp); fputc(!gopt, outfile); /* Graphics */
		fgetc(fp); fputc(nobeep,outfile); /* Quiet */
	/* Now the legs */
		while (	(i = fgetc(fp)) != EOF)
		{
			fputc(i, outfile);	
		}
	}	
	fclose(fp);	/* It's in the pot and ready! */
/*
 * nb: Because this is a one-pass transformation, the Z-code checksum is
 *    not set on the output file. If this causes trouble, a standalone
 *    checksum utility could be built.
 *
 */	

}
